;;; LispWorks 6.0 compatibility funtions and stubs for using on older LW versions
;;; LispWorks CAPI Reference Manual
;;; Copyright (c) 2009 by LispWorks Ltd.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Reimplementation pane-interface-* and active-pane-* families

(in-package :capi)

#-(or lispworks4 lispworks5)
(error "This code must not run on LispWorks 6.0 or later!")

(export '(capi::pane-descendant-child-with-focus))

(yl:export-toplevel (:capi)
  #:active-pane-copy
  #:active-pane-copy-p
  #:active-pane-cut
  #:active-pane-cut-p
  #:active-pane-deselect-all
  #:active-pane-deselect-all-p
  #:active-pane-paste
  #:active-pane-paste-p
  #:active-pane-select-all 
  #:active-pane-select-all-p 
  #:active-pane-undo 
  #:active-pane-undo-p     
  #:pane-interface-copy-object 
  #:pane-interface-copy-p 
  #:pane-interface-cut-object 
  #:pane-interface-cut-p 
  #:pane-interface-deselect-all 
  #:pane-interface-deselect-all-p 
  #:pane-interface-paste-object 
  #:pane-interface-paste-p 
  #:pane-interface-select-all 
  #:pane-interface-select-all-p 
  #:pane-interface-undo 
  #:pane-interface-undo-p)

(let ((lw:*handle-warn-on-redefinition* #+lispworks4 :quiet #-lispworks4 :warn))

;;; "Edit/select operations" and associated predicates for active pane.
;;; The operations call these generic functions when the active pane does
;;; not specify how to perform the operation.
;;; Args: pane      Active pane
;;;       interface Top-level interface of the pane.
;;; Predicate value: True if the operation can be performed.
;;; NB: Do not call these directly. 

(defgeneric pane-interface-cut-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))
(defgeneric pane-interface-cut-object (pane interface)
 (:method (pane interface)
  (declare (ignore interface))
  (ywi:beep-impossible pane :cut)))

(defgeneric pane-interface-copy-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))
(defgeneric pane-interface-copy-object (pane interface)
 (:method (pane interface)
  (declare (ignore interface))
  (ywi:beep-impossible pane :copy)))

(defgeneric pane-interface-paste-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))
(defgeneric pane-interface-paste-object (pane interface)
 (:method (pane interface)
  (declare (ignore interface))
  (ywi:beep-impossible pane :paste)))

(defgeneric pane-interface-undo-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))					; no way to determine for majority - may it be T?

(defgeneric pane-interface-undo (pane interface)
 (:method (pane interface)
  (declare (ignore pane))
  (ywi:beep-impossible pane :undo)))
  ;(ywi:beep
  ; #-Russian "Press CTRL+Z or try the Undo command from the context menu."
  ; #+Russian " ,  CTRL+Z   Undo   ."
  ; interface)))

(defgeneric pane-interface-select-all-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))
(defgeneric pane-interface-select-all (pane interface)
 (:method (pane interface)
  (declare (ignore interface))
  (ywi:beep-impossible pane :select-all)))

(defgeneric pane-interface-deselect-all-p (pane interface)
 (:method (pane interface)
  (declare (ignore pane interface))
  nil))
(defgeneric pane-interface-deselect-all (pane interface)
 (:method (pane interface)
  (declare (ignore interface))
  (ywi:beep-impossible pane :deselect-all)))

;;; These functions perform an "edit/select operation" on the
;;; active pane, or check if this operation is currently applicable.
;;;
;;; The active pane will be the one on the same screen as pane if
;;; pane is non-nil, or otherwise the same screen as the default interface.
;;;
;;; These functions find the active pane, that is the pane where
;;; keyboard input currently goes. Note that this is not necessarily
;;; a pane that is recognized by CAPI.
;;; Predicates return true if the operation is currently applicable.
;;; The other functions tell the active pane to do the operation.
;;; by means of the pane-interface-* generic functions.
;;; NB: Before calling a performing method, check with the corresponding *-p method
;;;	and beep as the edit-toolbar may act in passive mode.

(defun active-pane-cut-p (&optional pane)
  (multiple-value-call #'pane-interface-cut-p (ywi::get-active-pane-interface pane)))
(defun active-pane-cut (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-cut-p pane interface)
        (pane-interface-cut-object pane interface)
        (ywi:beep-impossible pane :cut))))

(defun active-pane-copy-p (&optional pane)
  (multiple-value-call #'pane-interface-copy-p (ywi::get-active-pane-interface pane)))
(defun active-pane-copy (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-copy-p pane interface)
        (pane-interface-copy-object pane interface)
        (ywi:beep-impossible pane :copy))))

(defun active-pane-paste-p (&optional pane)
  (multiple-value-call #'pane-interface-paste-p (ywi::get-active-pane-interface pane)))
(defun active-pane-paste (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-paste-p pane interface)
        (pane-interface-paste-object pane interface)
        (ywi:beep-impossible pane :paste))))

(defun active-pane-undo-p (&optional pane)
  (multiple-value-call #'pane-interface-undo-p (ywi::get-active-pane-interface pane)))
(defun active-pane-undo (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-undo-p pane interface)
        (pane-interface-undo pane interface)
        (ywi:beep-impossible pane :undo))))

(defun active-pane-select-all-p (&optional pane)
  (multiple-value-call #'pane-interface-select-all-p
    (ywi::get-active-pane-interface pane)))
(defun active-pane-select-all (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-select-all-p pane interface)
        (pane-interface-select-all pane interface)
        (ywi:beep-impossible pane :select-all))))

(defun active-pane-deselect-all-p (&optional pane)
  (multiple-value-call #'pane-interface-deselect-all-p
    (ywi::get-active-pane-interface pane)))
(defun active-pane-deselect-all (&optional pane)
  (multiple-value-bind (pane interface) (ywi::get-active-pane-interface pane)
    (if (pane-interface-deselect-all-p pane interface)
        (pane-interface-deselect-all pane interface)
        (ywi:beep-impossible pane :deselect-all))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  SIMPLE-PANE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod pane-interface-cut-p :around ((pane capi:simple-pane)
                                         (interface capi:interface))
  (and (capi:simple-pane-enabled pane) (call-next-method)))

(defmethod pane-interface-copy-p :around ((pane capi:simple-pane)
                                          (interface capi:interface))
  (and (capi:simple-pane-enabled pane) (call-next-method)))

(defmethod pane-interface-paste-p :around ((pane capi:simple-pane)
                                           (interface capi:interface))
  (and (capi:simple-pane-enabled pane) (call-next-method)))

(defmethod pane-interface-select-all-p :around ((pane capi:simple-pane)
                                                (interface capi:interface))
  (and (capi:simple-pane-enabled pane) (call-next-method)))

(defmethod pane-interface-deselect-all-p :around ((pane capi:simple-pane)
                                                  (interface capi:interface))
  (and (capi:simple-pane-enabled pane) (call-next-method)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  TEXT-INPUT-PANE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod pane-interface-cut-p ((pane capi:text-input-pane) (interface capi:interface))
  (declare (ignore interface))
  (and (capi:simple-pane-enabled pane)			; :read-only, :locked, or T
       (multiple-value-bind (start end) (capi:text-input-pane-selection pane)
         (not (eql start end)))))

(defmethod pane-interface-cut-object ((pane capi:text-input-pane)
                                      (interface capi:interface))
  #+win32
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_CUT-AUX handle)))				; -> change-callback
  #-win32							; cannot be undone
  (multiple-value-bind (start end) (capi:text-input-pane-selection pane)
     (if (eql start end)
         (ywi:beep-impossible pane :cut)
         (let* ((text (capi:text-input-pane-text pane))
                (selected (subseq text start end)))
           (capi:set-clipboard pane selected)
           (setf (ywi:pane-text pane)
                 (string-append (subseq text 0 start) (subseq text end)))))))

(defmethod pane-interface-copy-p ((pane capi:text-input-pane) (interface capi:interface))
  (declare (ignore interface))
  (multiple-value-bind (start end) (capi:text-input-pane-selection pane)
    (not (eql start end))))

(defmethod pane-interface-copy-object ((pane capi:text-input-pane)
                                       (interface capi:interface))
  (declare (ignore interface))
  #+win32
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_COPY-AUX handle)))				; no values
  #-win32
  (let ((selected (capi:text-input-pane-selected-text pane)))
    (if selected
        (capi:set-clipboard pane selected)
        (ywi:beep-impossible pane :copy))))

(defmethod pane-interface-paste-p ((pane capi:text-input-pane) (interface capi:interface))
  (declare (ignore interface))
  (let ((enabled (capi:simple-pane-enabled pane)))
    (and enabled (not (eq enabled :read-only)) (not (eq enabled :locked))
         (not (capi:clipboard-empty pane)))))

(defmethod pane-interface-paste-object ((pane capi:text-input-pane)
                                        (interface capi:interface))
  (declare (ignore interface))
  #+win32
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_PASTE-AUX handle)))				; -> change-callback
  #-win32							; cannot be undone
  (let ((string (capi:clipboard pane)))
    (if string
        (multiple-value-bind (start end) (capi:text-input-pane-selection pane)
          (let ((text (capi:text-input-pane-text pane))
                (position (+ start (length string))))
            (setf (ywi:pane-text pane)
                  (string-append (subseq text 0 start) string (subseq text end)))
            ;; Reposition caret just after the text pasted
            (capi:set-text-input-pane-selection pane position position)))
        (ywi:beep-impossible pane :paste))))

(defmethod pane-interface-select-all-p ((pane capi:text-input-pane)
                                        (interface capi:interface))
  (plusp (length (capi:text-input-pane-text pane))))

(defmethod pane-interface-select-all ((pane capi:text-input-pane)
                                      (interface capi:interface))
  (capi:set-text-input-pane-selection pane 0 (length (capi:text-input-pane-text pane))))

;;; Representation is an instance of capi-win32-lib::r-edit-text-input
#+win32
(defmethod pane-interface-undo-p ((pane capi:text-input-pane) (interface capi:interface))
  (declare (ignore interface))
  (let ((enabled (capi:simple-pane-enabled pane)))
    (when (and enabled (not (eq enabled :read-only)) (not (eq enabled :locked)))
      (let ((handle (capi:simple-pane-handle pane)))
        (and (yl:iplusp handle) (not (zerop (WIN32::EM_CANUNDO-AUX handle))))))))
#+win32
(defmethod pane-interface-undo ((pane capi:text-input-pane) (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (capi:simple-pane-handle pane)))
    (if (and (yl:iplusp handle) (eql (WIN32::EM_UNDO-AUX handle) 1))	; TRUE
        ;; Update the value returned by text-input-pane-* readers,
        ;; But seems not invoking change-callback
        (CAPI-WIN32-LIB::NOTE-R-EDIT-MAYBE-CHANGED (CAPI::REPRESENTATION pane))
        (ywi:beep-impossible pane :undo))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  TEXT-INPUT-CHOICE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; To undo, dive into the CAPI-WIN32-LIB::R-COMBOBOX-EDIT part of the widget
;;; representaion, the CAPI-WIN32-LIB::R-COMBOBOX-DROPDOWN.

;(defun WIN32::WM_UNDO-AUX (handle) (win32:send-message handle WIN32:WM_UNDO 0 0))

#+win32
(defmethod pane-interface-undo-p ((pane capi:text-input-choice) (interface capi:interface))
  (declare (ignore interface))
  (let ((enabled (capi:simple-pane-enabled pane)))
    (when (and enabled (not (eq enabled :read-only)) (not (eq enabled :locked)))
      (let ((handle (win32::R-WINDOW-HWND (slot-value (ywi::object-representation pane)
                                                      'CAPI-WIN32-LIB::EDIT))))
                                ;'WIN32::HWND)))
        (and (yl:iplusp handle) (not (zerop (WIN32::EM_CANUNDO-AUX handle))))))))

#+win32
(defmethod pane-interface-undo ((pane capi:text-input-choice) (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (win32::R-WINDOW-HWND (slot-value (ywi::object-representation pane)
                                                  'CAPI-WIN32-LIB::EDIT))))
                            ;'WIN32::HWND)))
    (if (and (yl:iplusp handle) (eql (WIN32::EM_UNDO-AUX handle) 1))	; TRUE
        t					; Q: does this invoke change-callback?
        (ywi:beep-impossible pane :undo))))

(lw:define-action "Delivery Actions" "Hack CAPI R-structures info"	; for slot-value
  (lambda () (pushnew 'CAPI-WIN32-LIB::R-COMBOBOX-DROPDOWN ;CAPI-WIN32-LIB::R-COMBOBOX-EDIT
                      (lw:delivery-value :symbols-to-keep-structure-info))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  TEXT-INPUT-RANGE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Representation and hanlde are only available through implementation
;;; Representation is an instance of capi-win32-lib::r-edit-spinbox

#+win32
(defmethod pane-interface-cut-p ((pane capi:text-input-range) (interface capi:interface))
  (declare (ignore pane interface))
  t)
#+win32
(defmethod pane-interface-cut-object ((pane capi:text-input-range)
                                      (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_CUT-AUX handle))))				; -> callback

#+win32
(defmethod pane-interface-copy-p ((pane capi:text-input-range) (interface capi:interface))
  (declare (ignore pane interface))
  t)
#+win32
(defmethod pane-interface-copy-object ((pane capi:text-input-range)
                                       (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_COPY-AUX handle))))				; no values

#+win32
(defmethod pane-interface-paste-p ((pane capi:text-input-range) (interface capi:interface))
  (declare (ignore pane interface))
  t)
#+win32
(defmethod pane-interface-paste-object ((pane capi:text-input-range)
                                        (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (ywi::object-handle pane)))
    (when (yl:iplusp handle)
      (WIN32::WM_PASTE-AUX handle))))				; -> callback

#+win32
(defmethod pane-interface-undo-p ((pane capi:text-input-range) (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (ywi::object-handle pane)))
    (and (yl:iplusp handle) (eql (WIN32::EM_CANUNDO-AUX handle) 1))))
#+win32
(defmethod pane-interface-undo ((pane capi:text-input-range) (interface capi:interface))
  (declare (ignore interface))
  (let ((handle (ywi::object-handle pane)))
    (if (and (yl:iplusp handle) (eql (WIN32::EM_UNDO-AUX handle) 1))	; TRUE
        ;; Sometimes does not invoke callback automatically - provide it
        (CAPI-WIN32-LIB::NOTE-R-EDIT-MAYBE-CHANGED (ywi::object-representation pane))
        (ywi:beep-impossible pane :undo))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  EDITOR-PANE  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod pane-interface-cut-p ((pane capi:editor-pane) (interface capi:interface))
  (capi:editor-pane-selected-text-p pane))	;(CAPI::EDITOR-PANE-CUT-P pane))
(defmethod pane-interface-cut-object ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Kill Region"))

(defmethod pane-interface-copy-p ((pane capi:editor-pane) (interface capi:interface))
  (capi:editor-pane-selected-text-p pane))	;(CAPI::EDITOR-PANE-COPY-P pane))
(defmethod pane-interface-copy-object ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Save Region"))		;"...Preserving Selection"

(defmethod pane-interface-paste-p ((pane capi:editor-pane) (interface capi:interface))
  (CAPI::EDITOR-PANE-PASTE-P pane))
(defmethod pane-interface-paste-object ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Un-Kill"))

(defmethod pane-interface-undo-p ((pane capi:editor-pane) (interface capi:interface))
  (let ((buffer (capi:editor-pane-buffer pane)))
    (and (editor:buffer-modified buffer) (editor:buffer-writable buffer))))
  ;(not (ywi:undo-buffer-empty-p (capi:editor-pane-buffer pane))))
(defmethod pane-interface-undo ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Undo"))

(defmethod pane-interface-select-all-p ((pane capi:editor-pane) (interface capi:interface))
  t)
(defmethod pane-interface-select-all ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Mark Whole Buffer"))

(defmethod pane-interface-deselect-all-p ((pane capi:editor-pane)(interface capi:interface))
  (editor:region-active-p (capi:editor-pane-buffer pane)))
(defmethod pane-interface-deselect-all ((pane capi:editor-pane) (interface capi:interface))
  (capi:call-editor pane "Pop Mark"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  LIST-PANEL  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defmethod pane-interface-select-all-p ((pane capi:list-panel) (interface capi:interface))
  (member (capi:choice-interaction pane) '(:multiple-selection :extended-selection)))

(defmethod pane-interface-select-all ((pane capi:list-panel) (interface capi:interface))
 ;;; Q: Shall we invoke selection-callback for each?
  (if (member (capi:choice-interaction pane) '(:multiple-selection :extended-selection))
      ;(and (null (capi:callbacks-selection-callback pane))
      ;     (null (capi:callbacks-retract-callback pane)))
      (setf (capi:choice-selection pane)
            (loop for i below (funcall (capi:collection-items-count-function pane)
                                       (capi:collection-items pane))
                  collect i))
      (call-next-method)))

) ;; End of lw:*handle-warn-on-redefinition*
