;;;-*- Mode: Lisp; Package: COMMON-LISP-USER -*-
;;; YstokGrid - LispWorks CAPI-based grid widget
;;; Copyright (c) 2003-2010 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Package definition

(in-package :cl-user)

(defpackage :ystok.widgets
 (:nicknames :ywi)
 (:use :common-lisp :ystok.library)
 (:documentation "CAPI-based GUI framework")
 (:export
  ;; General constants and vars
  #:+all-file-filters+
  #:+all-none-list+
  #:+big-h-gap+
  #:+big-v-gap+
  #:+h-gap+
  #:+items-uninitialized+
  #:+none-circle+
  #:+padding+
  #:+popup-args+
  #:+prompt-button-titles+
  #:+v-gap+
  #:*completion-gesture*
  #:*completion-help-key*
  #:*double-click-duration*
  #:*erroneous-pane*
  #:*hscroll-bar-height*
  #:*in-bulk*
  #:*message-area*
  #:*show-help-button*
  #:*show-tooltips*
  #:*vscroll-bar-width*
  #:*whitespace-bag*

  ;; Win32
  ;#+win32 "GetDoubleClickTime"

  ;; Macros, functions, and hacks
  #:active-pane-complete
  #:active-pane-complete-p
  #:aux-funcall
  #:beep
  #:beep-box
  #:beep-bulk
  #:bold-font
  #:changed
  #:complete-date
  #:confirm
  #:confirm-exit-application
  #:confirmer-create-callback
  #:current-top-level-interface
  #:define-icons
  #:disable
  #:display-errors
  #:enable
  #:enabled
  #:ensure-icons
  #:find-data-in-collection
  #:find-pane
  #:find-pane-by-name
  #:find-pane-to-focus
  ;#:find-pane-with-focus
  #:flash-window
  #:free-image
  #:hide
  #:icon-pane
  #:image
  #:imagep
  #:inverse-color
  #:insert-menu
  #:interface-owner
  #:help
  #:help-callback
  #:help-key-tooltip
  #:loaded
  #:merge-fonts
  #:message-area-format
  ;#:next-tab-page
  #:object-preference-path
  #:object-title
  #:object-title-gap
  #:object-title-pane
  #:pane-complete
  #:pane-complete-p
  #:pane-post-menu
  ;#:prior-tab-page
  #:prompt-for-number
  #:prompt-for-restart
  #:ring-collection-pushnew
  #:set-dropped-width
  #:show-raise-restore
  #:show
  #:shown
  #:slot-name
  #:string-hint-length
  #:top-level-hook
  ;#:undo-buffer-empty-p
  #:update-toolbars
  #:wait-execute-with-interface
  #:with-bulk

  ;; Cursors
  #:cursor-eql
  #:load-cursor-from-file
  #:restore-cursor
  #:set-cursor

  ;; Layout
  #:h-space
  #:v-space

  ;; Rectangle, mask, text
  #:draw-wrapped-text
  #:%font-ascent%
  #:%font-height%
  #:make-rect 
  #:rect-bind
  #:rect-left #:rect-top #:rect-width #:rect-height
  #:transform-mask
  #:translate-mask
  #:translate-point
  #:with-transformed-mask
  #:with-translated-mask

  ;; Progress dialog and meter
  #:*progress-dialog*
  #:display-progress-dialog
  #:progress-cancel
  #:progress-change
  #:progress-meter

  ;; Locale-specific pane dislayed content <-> stored object conversion
  #:$-princ-to-string-signed
  #:char-write-converter
  #:float-princ-to-string-d2
  #:float-princ-to-string-d2-signed
  #:float-princ-to-string-d3
  #:float-write-converter
  #:integer-write-converter
  #:real-princ-to-string
  #:real-princ-to-string-signed
  #:real-write-converter
  #:text-read-converter

  ;; Data and text lockable-pane
  #:change
  #:change-callback
  #:change-data
  #:changed-panes
  #:changed-slots
  #:changed-slots-alist
  #:clear
  #:data-check-button
  #:data-check-button-panel
  #:data-display-pane
  #:data-double-list-panel
  #:data-interface
  #:data-list-panel
  #:data-multi-line-pane
  #:data-multi-text-pane
  #:data-option-pane
  #:data-radio-button-panel
  #:data-reader
  #:data-text-choice
  #:data-text-input-range
  #:data-text-pane
  #:data-writer
  #:dynamic-complete
  #:interface-on-change
  #:load-pane
  #:load-panes
  ;#:lockable-text-choice
  ;#:lockable-text-pane
  #:mark-panes-saved
  #:+multi-line-tooltip-format+
  #:multi-line-list-read-converter
  #:multi-line-list-write-converter
  #:nullable-pane-text
  #:object-data
  #:on-change
  #:pane-text
  #:pane-slot-on-set
  #:dirty
  #:retrieval
  #:revert-panes
  #:save-pane
  #:save-panes
  #:set-pane-slot
  #:set-pane-slot-using-interface
  #:signal-choice-unmatched-error
  #:signal-not-nullable-error
  #:tracking-interface-mixin
  #:update-callback

  ;; editable-pinboard
  #:*chooser*
  #:current-displayer #:current-subscript
  #:displayer-rectangle
  #:editable-pinboard
  #:editable-pinboard-object-mixin
  ;#:editing-panes
  #:editing-pane-font
  #:editing-pane-mixin
  #:editing-pane-type
  #:ensure-editing-collection-items
  #:ensure-editing-pane
  #:enter-editing-pane
  #:load-editing-pane
  #:hide-editing-panes
  #:make-editing-pane
  #:overlay
  #:pane-args
  #:pinboard-font
  #:read-displayer-data
  #:save-editing-pane
  #:unload-editing-pane
  #:write-displayer-data

  ;; Framework generics, menus, and toolbars
  #:about-application
  #:beep-impossible
  #:beep-not-implemented
  #:*doc-image-set* #:doc-image-set
  #:doc-print-mc
  #:doc-print-tc
  #:doc-refresh-mi
  #:doc-refresh-tc
  ;#:*edit-image-set*
  #:edit-menu
  #:edit-toolbar
  #:interface-confirm-destroy
  #:interface-on-create
  #:interface-on-destroy
  #:make-edit-toolbar
  #:operate-on-focus
  #:operate-on-interface
  #:save-interface))
