;;;-*- Mode: Lisp; -*-
;;; Ystok-Widgets - LispWorks CAPI-based GUI framework
;;; Copyright (c) 2003-2010 Dr. Dmitriy Ivanov. All rights reserved.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Text draw helpers

(in-package :ystok.widgets)

(defgeneric %font-ascent% (output-pane)
 (:method ((port gp:graphics-port-mixin))
  (gp:get-font-ascent port)))

(defgeneric %font-height% (output-pane)
 (:method ((port gp:graphics-port-mixin))
  (gp:get-font-height port)))

(defun draw-wrapped-text (pinboard wrapped-text port left top width height
                          &optional (x-align :left) (y-align :top)
                                    graphics-args)
 ;;;  Draw text within graphic mask established in port
  ;; Args: wrapped-text : String or list of strings, perhaps empty
  ;;       port : Either coincides with grid-component or is a printer port.
  ;;	   left,top,width,heigth  Drawing area internal coordinates and size, better 
  ;;				  with additional 1 pixel of horizontal padding
  ;; NB: Call %font-height% and %font-ascent% on pinboard or port?
  (declare (fixnum left width height)
           #-debug (optimize (speed 3) (safety 0) (hcl:fixnum-safety 0)))
  (when wrapped-text
    (let* ((font (getf graphics-args :font))
                     ;(capi:simple-pane-font pinboard)))
           (font-ascent (if font (gp:get-font-ascent port font)(%font-ascent% pinboard)))
           (font-height (the fixnum (1- (if font	;condense a bit
                                            (gp:get-font-height port font)
                                            (%font-height% pinboard))))))
      (declare (fixnum font-ascent font-height))
      ;; Draw substrings stored in wrapped-cache, the first one is always drawn.
      (if (listp wrapped-text)
          (loop for dy fixnum upfrom
                    (+ font-ascent 
                       (ecase y-align
                         (:top 0)
                         (:center (the fixnum (max (values (floor (the fixnum
                                                   (- height (* font-height
                                                                (length wrapped-text))))
                                                                  2))
                                                   -2)))
                         (:bottom (the fixnum (- height (* font-height
                                                           (length wrapped-text)))))))
                    by font-height
                and substring in wrapped-text
                do (apply 'gp:draw-string port substring
                          (ecase x-align
                            (:left left)
                            (:center
                             (+ left (the fixnum	 ; required-width
                                          (max (values (floor (the fixnum (- width
                                               #1=(nth-value 2 (gp:get-string-extent
                                                                port substring font))))
                                                                 2)) 
                                               -1))))
                            (:right (+ left (the fixnum (- width #1# 2)))))
                          (+ top dy)
                          graphics-args)
                until (>= dy height))
          ;; else (stringp wrapped-text)
          (apply 'gp:draw-string port wrapped-text
                 (ecase x-align
                   (:left left)
                   (:center				; required-width
                    (+ left (the fixnum (max (values (floor (the fixnum (- width
                                       #2=(nth-value 2 (gp:get-string-extent
                                                        port wrapped-text font)))) 2))
                                             -1))))
                   (:right (+ left (the fixnum (- width #2# 2)))))
                 (+ top font-ascent
                    (ecase y-align
                      (:top 0)
                      (:center (the fixnum (max (values (floor (the fixnum 
                                                         (- height font-height))
                                                                  2))
                                                -2)))
                      (:bottom (the fixnum (- height font-height)))))
                 graphics-args) )))
  (values))

(defun draw-button-text (port text)
 ;;; Center the text vertically and horizontally
  (multiple-value-bind (width height) (capi:simple-pane-visible-size port)
    (declare (fixnum width height))
    (let ((y (the fixnum (+ (the fixnum (values (truncate (the fixnum
                                                           (- height (the fixnum
                                                            (gp:get-font-height port))))
                                                          2)))
                            (the fixnum (gp:get-font-ascent port))))))
      (if (characterp text)
          (gp:draw-character port text (the fixnum (values (truncate width 2))) y)
          (gp:draw-string port text 
               (the fixnum (values (truncate (the fixnum (- width (the fixnum
                                                     (nth-value 2 (gp:get-string-extent
                                                                   port text)))))
                                             2)))
               y)))))

(defun max-width-in-characters (text)
 ;;; TODO: &optional pane - if supplied, compute exact width in pixels
  (loop for arg in (split-seq #\Linefeed text) maximize (length arg)))

(defun max-height-in-characters (text)
  (1+ (count #\Linefeed text)))
